package com.fuint.common.service;

import com.baomidou.mybatisplus.extension.service.IService;
import com.fuint.common.dto.ext.CountryDto;
import com.fuint.common.vo.CountryVo;
import com.fuint.framework.pagination.PaginationRequest;
import com.fuint.framework.pagination.PaginationResponse;
import com.fuint.repository.model.MtCountry;
import com.fuint.framework.exception.BusinessCheckException;
import org.springframework.web.bind.annotation.RequestParam;

import java.util.List;
import java.util.Map;

/**
 * 国家表业务接口
 *
 * Created by pzg
 * CopyRight https://www.fuint.cn
 */
public interface CountryService extends IService<MtCountry> {

    /**
     * 分页查询列表
     *
     * @param paginationRequest
     * @return
     */
    PaginationResponse<MtCountry> queryCountryListByPagination(PaginationRequest paginationRequest) throws BusinessCheckException;

    /**
     * 添加国家表
     *
     * @param  mtCountry
     * @throws BusinessCheckException
     * @return
     */
    MtCountry addCountry(MtCountry mtCountry) throws BusinessCheckException;

    /**
     * 根据ID获取国家表信息
     *
     * @param id ID
     * @throws BusinessCheckException
     * @return
     */
    MtCountry queryCountryById(Long id) throws BusinessCheckException;

    /**
     * 根据ID删除国家表
     *
     * @param id ID
     * @throws BusinessCheckException
     * @return
     */
    boolean deleteCountry(Long id) throws BusinessCheckException;

    /**
     * 更新国家表
     * @param  countryDto
     * @throws BusinessCheckException
     * @return
     * */
    boolean updateCountry(CountryDto countryDto,Long accountId) throws BusinessCheckException;

    /**
     * 根据条件搜索国家表
     *
     * @param params 查询参数
     * @throws BusinessCheckException
     * @return
     * */
    List<MtCountry> queryCountryListByParams(Map<String, Object> params) throws BusinessCheckException;

    /**
     * 新增国家
     * @param countryDto
     * @param accountInfoId
     * @return
     */
	boolean saveCountry(CountryDto countryDto, Long accountInfoId) throws BusinessCheckException;

    List<CountryVo> getCountryTree(String countryCode, String countryName);
}
