package com.fuint.common.service;

import com.baomidou.mybatisplus.extension.service.IService;
import com.fuint.common.dto.IngredientDto;
import com.fuint.common.vo.IngredientVo;
import com.fuint.framework.pagination.PaginationRequest;
import com.fuint.framework.pagination.PaginationResponse;
import com.fuint.framework.web.PageResult;
import com.fuint.repository.model.MtIngredient;
import com.fuint.framework.exception.BusinessCheckException;
import java.util.List;
import java.util.Map;

/**
 * 菜品配料表业务接口
 *
 * Created by pzg
 * CopyRight https://www.fuint.cn
 */
public interface IngredientService extends IService<MtIngredient> {

    /**
     * 分页查询列表
     *
     * @param ingredientDto
     * @return
     */
    PageResult<IngredientVo> queryIngredientPageList(IngredientDto ingredientDto);

    /**
     * 添加菜品配料表
     *
     * @param  ingredientDtoList
     * @throws BusinessCheckException
     * @return
     */
    void addIngredientForGoods(List<IngredientDto> ingredientDtoList) throws BusinessCheckException;

    /**
     * 根据ID获取菜品配料表信息
     *
     * @param id ID
     * @throws BusinessCheckException
     * @return
     */
    IngredientVo queryIngredientById(Long id) throws BusinessCheckException;

    /**
     * 更新菜品配料表
     * @param  ingredientDto
     * @throws BusinessCheckException
     * @return
     * */
    IngredientVo updateIngredient(IngredientDto ingredientDto) throws BusinessCheckException;

    /**
     * 根据ID删除菜品配料表
     *
     * @param id ID
     * @throws BusinessCheckException
     * @return
     */
    void deleteIngredientById(Long id);


}
